#ifndef MPEG3PROTOS_H
#define MPEG3PROTOS_H

/* CSS */

mpeg3_css_t* mpeg3_new_css();
int mpeg3_delete_css(mpeg3_css_t *css);
int mpeg3_get_keys(mpeg3_css_t *css, char *path);
int mpeg3_decrypt_packet(mpeg3_css_t *css, unsigned char *sector);

/* DEMUX */

mpeg3_demuxer_t* mpeg3_new_demuxer(mpeg3_t *file, int do_audio, int do_video, int stream_id);
void mpeg3_delete_demuxer(mpeg3_demuxer_t *demuxer);
int mpeg3demux_read_data(mpeg3_demuxer_t *demuxer, unsigned char *output, long size);
unsigned int mpeg3demux_read_int32(mpeg3_demuxer_t *demuxer);
unsigned int mpeg3demux_read_int24(mpeg3_demuxer_t *demuxer);
unsigned int mpeg3demux_read_int16(mpeg3_demuxer_t *demuxer);
double mpeg3demux_length(mpeg3_demuxer_t *demuxer);
mpeg3_demuxer_t* mpeg3_get_demuxer(mpeg3_t *file);
long mpeg3demux_tell(mpeg3_demuxer_t *demuxer);
double mpeg3demux_tell_percentage(mpeg3_demuxer_t *demuxer);
double mpeg3demux_get_time(mpeg3_demuxer_t *demuxer);
int mpeg3demux_eof(mpeg3_demuxer_t *demuxer);
int mpeg3demux_bof(mpeg3_demuxer_t *demuxer);

int mpeg3demux_create_title(mpeg3_demuxer_t *demuxer, int timecode_search, FILE *toc);
int mpeg3demux_print_timecodes(mpeg3_title_t *title, FILE *output);
int mpeg3demux_read_titles(mpeg3_demuxer_t *demuxer);
int mpeg3demux_copy_titles(mpeg3_demuxer_t *dst, mpeg3_demuxer_t *src);
int mpeg3demux_print_streams(mpeg3_demuxer_t *demuxer, FILE *toc);
double mpeg3demux_length(mpeg3_demuxer_t *demuxer);
int mpeg3demux_eof(mpeg3_demuxer_t *demuxer);
int mpeg3demux_bof(mpeg3_demuxer_t *demuxer);
int mpeg3demux_seek_byte(mpeg3_demuxer_t *demuxer, long byte);
int mpeg3demux_seek_time(mpeg3_demuxer_t *demuxer, double new_time);
int mpeg3demux_seek_percentage(mpeg3_demuxer_t *demuxer, double percentage);
double mpeg3demux_tell_percentage(mpeg3_demuxer_t *demuxer);
double mpeg3demux_get_time(mpeg3_demuxer_t *demuxer);
long mpeg3demux_tell(mpeg3_demuxer_t *demuxer);
long mpeg3demuxer_total_bytes(mpeg3_demuxer_t *demuxer);
mpeg3_demuxer_t* mpeg3_get_demuxer(mpeg3_t *file);

mpeg3demux_timecode_t* mpeg3demux_next_timecode(mpeg3_demuxer_t *demuxer, 
		int *current_title, 
		int *current_timecode,
		int current_program);
mpeg3demux_timecode_t* mpeg3demux_prev_timecode(mpeg3_demuxer_t *demuxer, 
		int *current_title, 
		int *current_timecode,
		int current_program);
int mpeg3demux_open_title(mpeg3_demuxer_t *demuxer, int title_number);
int mpeg3demux_assign_programs(mpeg3_demuxer_t *demuxer);


/* TITLE */

mpeg3_title_t* mpeg3_new_title(mpeg3_t *file, char *path);
void mpeg3_delete_title(mpeg3_title_t *title);
void mpeg3_copy_title(mpeg3_title_t *dst, mpeg3_title_t *src);
void mpeg3_dump_title(mpeg3_title_t *title);


/* ATRACK */

mpeg3_atrack_t* mpeg3_new_atrack(mpeg3_t *file, int stream_id, int is_ac3, mpeg3_demuxer_t *demuxer);
void mpeg3_delete_atrack(mpeg3_t *file, mpeg3_atrack_t *atrack);

/* VTRACK */

mpeg3_vtrack_t* mpeg3_new_vtrack(mpeg3_t *file, int stream_id, mpeg3_demuxer_t *demuxer);
void mpeg3_delete_vtrack(mpeg3_t *file, mpeg3_vtrack_t *vtrack);

/* AUDIO */
mpeg3audio_t* mpeg3audio_new(mpeg3_t *file, mpeg3_atrack_t *track, int is_ac3);
void mpeg3audio_delete(mpeg3audio_t *audio);
int mpeg3audio_seek_percentage(mpeg3audio_t *audio, double percentage);
int mpeg3audio_seek_sample(mpeg3audio_t *audio, long sample);
int mpeg3audio_read_raw(mpeg3audio_t *audio, unsigned char *output, long *size, long max_size);
int mpeg3audio_decode_audio(mpeg3audio_t *audio, float *output_f, short *output_i, int channel, long start_position, long len);


/* VIDEO */
mpeg3video_t* mpeg3video_new(mpeg3_t *file, mpeg3_vtrack_t *track);
int mpeg3video_delete(mpeg3video_t *video);
int mpeg3video_seek_percentage(mpeg3video_t *video, double percentage);
int mpeg3video_previous_frame(mpeg3video_t *video);
int mpeg3video_seek_frame(mpeg3video_t *video, long frame);
int mpeg3video_read_raw(mpeg3video_t *video, unsigned char *output, long *size, long max_size);
int mpeg3video_read_frame(mpeg3video_t *video, 
		long frame_number, 
		unsigned char **output_rows,
		int in_x, 
		int in_y, 
		int in_w, 
		int in_h, 
		int out_w, 
		int out_h, 
		int color_model);
int mpeg3video_read_yuvframe(mpeg3video_t *video, 
					long frame_number, 
					char *y_output,
					char *u_output,
					char *v_output,
					int in_x,
					int in_y,
					int in_w,
					int in_h);

int mpeg3video_drop_frames(mpeg3video_t *video, long frames);


/* FILESYSTEM */

mpeg3_fs_t* mpeg3_new_fs(char *path);
int mpeg3_delete_fs(mpeg3_fs_t *fs);
int mpeg3_copy_fs(mpeg3_fs_t *dst, mpeg3_fs_t *src);
long mpeg3io_get_total_bytes(mpeg3_fs_t *fs);
int mpeg3io_open_file(mpeg3_fs_t *fs);
int mpeg3io_close_file(mpeg3_fs_t *fs);
int mpeg3io_read_data(unsigned char *buffer, long bytes, mpeg3_fs_t *fs);
int mpeg3io_device(char *path, char *device);
int mpeg3io_seek(mpeg3_fs_t *fs, long byte);
int mpeg3io_seek_relative(mpeg3_fs_t *fs, long bytes);

/* BITSTREAM */
mpeg3_bits_t* mpeg3bits_new_stream(mpeg3_t *file, mpeg3_demuxer_t *demuxer);
void mpeg3bits_delete_stream(mpeg3_bits_t* stream);
int mpeg3bits_read_buffer(mpeg3_bits_t* stream, unsigned char *buffer, int bytes);
int mpeg3bits_use_ptr(mpeg3_bits_t* stream, unsigned char *buffer);
int mpeg3bits_use_demuxer(mpeg3_bits_t* stream);
void mpeg3bits_start_reverse(mpeg3_bits_t* stream);
void mpeg3bits_start_forward(mpeg3_bits_t* stream);
int mpeg3bits_refill(mpeg3_bits_t* stream);
int mpeg3bits_refill_backwards(mpeg3_bits_t* stream);
int mpeg3bits_byte_align(mpeg3_bits_t *stream);
int mpeg3bits_seek_end(mpeg3_bits_t* stream);
int mpeg3bits_seek_start(mpeg3_bits_t* stream);
int mpeg3bits_seek_time(mpeg3_bits_t* stream, double time_position);
int mpeg3bits_seek_byte(mpeg3_bits_t* stream, long position);
int mpeg3bits_seek_percentage(mpeg3_bits_t* stream, double percentage);
int mpeg3bits_tell(mpeg3_bits_t* stream);
int mpeg3bits_getbitoffset(mpeg3_bits_t *stream);
unsigned int mpeg3bits_getbits(mpeg3_bits_t* stream, int n);



/* MISC */
int mpeg3_mmx_test();

/* LIBMPEG3 */
int mpeg3_read_toc(mpeg3_t *file);
unsigned long mpeg3_get_timestamp(mpeg3_demuxer_t *demuxer);


#endif
